package util

import (
	"fmt"
	"io"
	"strings"
)

type MessageManager struct {
	/* listeners is an array of functions that
	 * tell the manager how to listen for specific
	 * text and what to do if we receive it.
	 * Each listener returns true if the input was
	 * "consumed" (i.e. - Don't send to any more
	 * listeners)
	 */
	listeners []func(inp *Message) bool

	/* tempListeners is an array of functions that
	 * work the same as 'listeners', but these are
	 * just temporary and higher priority than
	 * 'listeners'
	 */
	tempListeners []func(inp *Message) bool

	/* finishedTempListener is an index of a
	 * tempListener to be removed
	 */
	finishedTempListener int

	output io.WriteCloser
}

func (mm MessageManager) Output(o string) {
	if !strings.HasSuffix("\n", o) {
		o = o + "\n"
	}
	mm.output.Write([]byte(o))
}

func (mm MessageManager) ProcessMessage(inp string) bool {
	// First of all, create the message from inp
	m := NewMessage(inp)
	tempListeners := mm.tempListeners
	listeners := mm.listeners
	fmt.Printf("[%d]l, [%d]tmpl: Processing Message: >>> %s\n", len(listeners), len(tempListeners), strings.Replace(inp, "\n", "", -1))
	// Now run the message through all of mm's tempListeners
	for i := range tempListeners {
		fmt.Printf("Processing message in tempListener %d\n", i)
		// Pop the listener off of the stack
		consumed := tempListeners[i](m)
		if consumed {
			// When a temp listener is consumed, we delete it
			fmt.Println("Consumed Event from Temporary Listener.")
			mm.RemoveTempListener(i)
			return true
		}
	}
	// and run the message through all of mm's listeners
	for i := range listeners {
		fmt.Printf("Processing message in listener %d\n", i)
		consumed := listeners[i](m)
		if consumed {
			return true
		}
	}
	// Message not consumed, return false
	return false
}

func (mm MessageManager) AddListener(l func(*Message) bool) {
	mm.listeners = append(mm.listeners, l)
}

func (mm MessageManager) AddTempListener(l func(*Message) bool) {
	mm.tempListeners = append(mm.tempListeners, l)
}

func (mm MessageManager) RemoveListener(i int) {
	if i > -1 && i < len(mm.listeners) {
		mm.listeners = append(mm.listeners[:i], mm.listeners[i+1:]...)
	}
}
func (mm MessageManager) RemoveTempListener(i int) {
	if i > -1 && i < len(mm.tempListeners) {
		fmt.Printf("Removing Temp Listener %d\n", i)
		fmt.Printf("Before: %d\n", len(mm.tempListeners))
		mm.tempListeners = append(mm.tempListeners[:i], mm.tempListeners[i+1:]...)
		fmt.Printf("After: %d\n", len(mm.tempListeners))
	}
}

func NewManager(o io.WriteCloser) MessageManager {
	mm := MessageManager{
		//		listeners:     make(([]func(*Message) bool), 0, 10),
		//		tempListeners: make(([]func(*Message) bool), 0, 10),
		finishedTempListener: -1,
		output:               o,
	}
	return mm
}
